/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

if (typeof Cc == 'undefined') var Cc = Components.classes;
if (typeof Ci == 'undefined') var Ci = Components.interfaces;
if (typeof Cu == "undefined") var Cu = Components.utils;
if (typeof Cr == 'undefined') var Cr = Components.results;


/**
 * Display debug messages on error console
 *
 **/
if (typeof(DaapDebug) == 'undefined') {
  function DaapDebug(aMessage, aStart) {
    if (aStart) {
      var elapsed = Date.now() - aStart;
      var time = (elapsed / 1000);
      aMessage += " (time: " + time + ")";
    }
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
    consoleService.logStringMessage("[Daap Client] " + aMessage);
  }
}



/* Extension ID */
const EXTENSION_UUID = "daapclient@songbirdnest.com";

const PROPERTY_FILE = "chrome://daapclient/locale/daapclient.properties";


/* Make sure we have the javascript modules we're going to use */
if (typeof(XPCOMUtils) == "undefined") {
  Cu.import("resource://gre/modules/XPCOMUtils.jsm");
  if (!XPCOMUtils)
    throw new Error("Import of XPCOMUtils module failed!");
}


/* Make a namespace. */
if (typeof DaapClient == 'undefined') {
  var DaapClient = {};
}

if (!(daapLibManager instanceof Ci.sbIDaapLibraryManager)) {
  var daapLibManager = Cc["@songbirdnest.com/daap/library-manager;1"]
                        .getService(Ci.sbIDaapLibraryManager);
}

/**
 * UI controller that is loaded into the main player window
 */
DaapClient.Controller = {

  /**
   * Called when the window finishes loading
   */
  onLoad: function() {
    // initialization code
    this._initialized = true;
    
    // Make a local variable for this controller so that
    // it is easy to access from closures.
    var controller = this;

    /* check if the gstreamer daap protocol handler is loaded */
    if (this.checkGstreamer()) {

      /* enable only if it isn't the first run */
      daapLibManager.init();
      DaapDiscover.init();
      DaapLoadTracks.init();

      // Attach doNew command
      this._newCmd = document.getElementById("daapclient-new-cmd");
      this._newCmd.addEventListener("command", 
                                    function() { controller.doNew(); }, 
                                    false);
    }
    else {
      /* TODO: translate */
      if(DaapDebug) DaapDebug("checkGstreamer failed");
      alert("Error:\nGstreamer daap protocol handler not found!\nDaapclient is disabled.");
    }

    // Perform extra actions the first time the extension is run
    //if (!this.checkFirstRun()) {
    //}

  },
  
  /**
   * Called when the window is about to close
   */
  onUnLoad: function() {
    this._initialized = false;

    /* close all sessions */
    let sessionManager = Cc["@songbirdnest.com/daap/session-manager;1"]
                           .getService(Ci.sbIDaapSessionManager);

    try { 
      sessionManager.closeAll();
    }
    catch(e) {};
    
    DaapLoadTracks.shutdown();
    DaapDiscover.shutdown();
    daapLibManager.shutdown();
  },

  //!/ /**
  //!/  * Awk for restarting
  //!/  */
  //!/ restartWarning : function() {
  //!/   l10n = Cc["@mozilla.org/intl/stringbundle;1"]
  //!/           .getService(Ci.nsIStringBundleService)
  //!/           .createBundle(PROPERTY_FILE);
  //!/   

  //!/   alert(l10n.GetStringFromName("RestartedRequired"));
  //!/ },

  //!/ /**
  //!/  * Perform extra setup the first time the extension is run
  //!/  */
  //!/ firstRunSetup : function() {
  //!/   /* delete the gstreamer registry so the daap handler is added upon rebuild */
  //!/   let file = Cc["@mozilla.org/file/directory_service;1"]
  //!/          			 .createInstance(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
  //!/   file.append("gstreamer-0.10");
  //!/   file.append("registry.bin");
  //!/   if (file.exists()) {
  //!/     file.remove(false);
  //!/     // Warn the user about restarting Songbird
  //!/     setTimeout(function(controller) { controller.restartWarning(); }, 1000, this); 
  //!/   }
  //!/ }, //firstRunSetup


  //!/ checkFirstRun: function ()
  //!/ {
  //!/   //  from https://developer.mozilla.org/en/Code_snippets/On_page_load#Running_code_on_an_extension%27s_first_run_or_after_an_extension%27s_update
  //!/   
  //!/   var Prefs = Components.classes["@mozilla.org/preferences-service;1"]
  //!/                  .getService(Components.interfaces.nsIPrefService);
  //!/   Prefs = Prefs.getBranch("extensions.daapclient.");
  //!/   


  //!/   var ver = -1, firstrun = true;

  //!/   var gExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
  //!/                           .getService(Components.interfaces.nsIExtensionManager);
  //!/   var current = gExtensionManager.getItemForID(EXTENSION_UUID).version;
  //!/   //gets the version number.
	//!/ 	
  //!/   try {
  //!/     ver = Prefs.getCharPref("version");
  //!/   	firstrun = Prefs.getBoolPref("firstrun");
  //!/   } catch (e) {
  //!/     //nothing
  //!/   } finally {
  //!/     if (firstrun) {
  //!/       Prefs.setBoolPref("firstrun",false);
  //!/       Prefs.setCharPref("version",current);
	//!/ 
  //!/       // Insert code for first run here        
  //!/       this.firstRunSetup();
  //!/       return true;
  //!/     }		
  //!/     
  //!/     if (ver!=current && !firstrun){ // !firstrun ensures that this section does not get loaded if its a first run.
  //!/       Prefs.setCharPref("version",current);
  //!/       
  //!/       // Insert code if version is different here => upgrade
  //!/       this.firstRunSetup();
  //!/       return true;
  //!/     }

  //!/   } //try

  //!/   return false;
  //!/ }, //checkFirstRun


  /* checkGstreamer: check if gstreamer can play daap:// URI's
   *
   * \return boolean
   *
   **/
  checkGstreamer: function() 
  {
    let gs = Cc["@songbirdnest.com/daap/gstreamer;1"]
              .createInstance(Ci.sbIDaapGstreamerAccess);

    if (!gs) {
      /* error: GstreamerAccess class not found */
      if(DaapDebug) DaapDebug("GstreamerAccess class not found");
      return false;
    }

    if (gs.isProtocolSupported("daap")) {
      /* ok */
      return true;
    }

    /* delete the registry so it is rebuild on the next start */
    if (gs.deleteRegistryCache()) {
      if(DaapDebug) DaapDebug("gstreamer registry file deleted");
    }
    else if(DaapDebug) DaapDebug("could not delete gstreamer registry file");

    /* try to reload the mozilla plugin */
    if (gs.reloadPlugin("mozilla")) {
      if(DaapDebug) DaapDebug("mozilla plugin reload ok");
      return gs.isProtocolSupported("daap");
    }

    if(DaapDebug) DaapDebug("could not reload mozilla plugin");

    /* fail, nothing to do */
    return false;
  }, //checkGstreamer


  /**
   * Opens new daap library dialog
   */
  doNew: function() 
  {
    let params = null;
    let parentWindow = null;

    // Open the window
    SBOpenModalDialog("chrome://daapclient/content/dialognew.xul",
                      "",
                      "chrome,centerscreen",
                      params,
                      parentWindow);
  }, //doNew

};

window.addEventListener("load", function(e) { DaapClient.Controller.onLoad(e); }, false);
window.addEventListener("unload", function(e) { DaapClient.Controller.onUnLoad(e); }, false);
